@@ -2,8 +2,8 @@
 """
 @Author: ChenXiaolei
 @Date: 2020-03-06 23:17:54
-@LastEditTime: 2020-05-11 22:17:21
-@LastEditors: ChenXiaolei
+@LastEditTime: ,: 2020-10-21 16:13:14
+@LastEditors: ,: ChenXiaolei
 @Description: 数据库基础操作类
 """
 
@@ -43,7 +43,9 @@ class BaseModel:
         elif isinstance(field_value, datetime.date):
             return field_value.strftime("%Y-%m-%d")
         elif isinstance(field_value, bytes):
-            return ord(field_value)
+            return field_value.decode()
+        elif isinstance(field_value, decimal.Decimal):
+            return str(decimal.Decimal(field_value).quantize(decimal.Decimal('0.00')))
         return field_value
 
     def __row_entity(self, data):
@@ -130,12 +132,22 @@ class BaseModel:
         if where and where.strip() != '':
             where = " WHERE " + where
         if group_by and group_by.strip() != '':
-            group_by = " GROUP BY "+group_by
+            group_by = " GROUP BY " + group_by
         if order_by and order_by.strip() != '':
             order_by = " ORDER BY " + order_by
+
         sql = f"SELECT {field} FROM {self.table_name}{where}{group_by}{order_by} LIMIT {str(int(page_index) * int(page_size))},{str(page_size)};"
         list_row = self.db.fetch_all_rows(sql, params)
-        return self.__row_entity_list(list_row)
+
+        sql = f"SELECT COUNT({self.primary_key_field}) AS count FROM {self.table_name}{where}{group_by};"
+        row = self.db.fetch_one_row(sql, params)
+
+        if row and 'count' in row and int(row['count']) > 0:
+            row_count = int(row["count"])
+        else:
+            row_count = 0
+
+        return self.__row_entity_list(list_row), row_count
 
     def get_entity(self, where='', group_by='', order_by='',  params=None):
         """
@@ -165,7 +177,7 @@ class BaseModel:
         @last_editors: ChenXiaolei
         """
         sql = f"SELECT * FROM {self.table_name} WHERE {self.primary_key_field}=%s;"
-        list_row = self.db.fetch_one_row(sql, primary_key_id)
+        list_row = self.db.fetch_one_row(sql, [primary_key_id])
         return self.__row_entity(list_row)
 
     def get_total(self, where='', group_by='', field=None, params=None):
@@ -199,11 +211,19 @@ class BaseModel:
         if where:
             where = " WHERE " + where
             sql = "DELETE FROM " + self.table_name + where
-            list_row = self.db.update(sql, params)
-            if list_row is not None and list_row.rowcount >= 0:
+
+            if self.is_transaction():
+                transaction_item = {}
+                transaction_item["sql"] = sql
+                transaction_item["params"] = params
+                self.db_transaction.transaction_list.append(transaction_item)
                 return True
             else:
-                return False
+                list_row = self.db.delete(sql, params)
+                if list_row is not None and list_row > 0:
+                    return True
+                else:
+                    return False
 
     def add_entity(self, model, ignore=False):
         """
@@ -219,16 +239,24 @@ class BaseModel:
         insert_value_str = ""
         param = []
         for field_str in field_list:
-            if str(field_str).lower() == self.primary_key_field:
+            param_value = str(getattr(model, field_str))
+            if str(field_str).lower() == self.primary_key_field.lower() and param_value == "0":
                 continue
             insert_field_str += str(field_str + ",")
             insert_value_str += "%s,"
-            param.append(str(getattr(model, field_str)))
+            param.append(param_value)
         insert_field_str = insert_field_str.rstrip(',')
         insert_value_str = insert_value_str.rstrip(',')
         sql = f"INSERT{' IGNORE' if ignore else ''} INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str});"
-        list_row = self.db.insert(sql, tuple(param))
-        return list_row
+        if self.is_transaction():
+            transaction_item = {}
+            transaction_item["sql"] = sql
+            transaction_item["params"] = tuple(param)
+            self.db_transaction.transaction_list.append(transaction_item)
+            return True
+        else:
+            list_row = self.db.insert(sql, tuple(param))
+            return list_row
 
     def add_list(self, model_list, ignore=False, transaction=False):
         """
@@ -250,19 +278,26 @@ class BaseModel:
             insert_value_str = ""
             param = []
             for field_str in field_list:
-                if str(field_str).lower() == self.primary_key_field:
+                param_value = str(getattr(model, field_str))
+                if str(field_str).lower() == self.primary_key_field.lower() and param_value == "0":
                     continue
                 insert_field_str += str(field_str + ",")
                 insert_value_str += "%s,"
-                param.append(str(getattr(model, field_str)))
+                param.append(param_value)
             insert_field_str = insert_field_str.rstrip(',')
             insert_value_str = insert_value_str.rstrip(',')
             sql = f"INSERT{' IGNORE' if ignore else ''} INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str});"
-            if not transaction:
-                self.db.insert(sql, tuple(param))
+
+            sql_item = {}
+            sql_item["sql"] = sql
+            sql_item["params"] = tuple(param)
+            if self.is_transaction():
+                self.db_transaction.transaction_list.append(sql_item)
+            elif transaction:
+                sql_list.append(sql_item)
             else:
-                sql_list.append(sql)
-        if transaction:
+                self.db.insert(sql, tuple(param))
+        if transaction and not self.is_transaction():
             if not self.transaction_execute(sql_list):
                 return False
         return True
@@ -283,27 +318,38 @@ class BaseModel:
         insert_value_str = ""
         param = []
         for field_str in field_list:
-            if str(field_str).lower() == self.primary_key_field:
+            param_value = str(getattr(model, field_str))
+            if str(field_str).lower() == self.primary_key_field.lower() and param_value == "0":
                 continue
             insert_field_str += str(field_str + ",")
             insert_value_str += "%s,"
-            param.append(str(getattr(model, field_str)))
+            param.append(param_value)
         if params:
             param = param+params
         insert_field_str = insert_field_str.rstrip(',')
         insert_value_str = insert_value_str.rstrip(',')
         sql = f"INSERT INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str}) ON DUPLICATE KEY UPDATE {update_sql};"
-        list_row = self.db.insert(sql, tuple(param))
-        return list_row
+        if self.is_transaction():
+            transaction_item = {}
+            transaction_item["sql"] = sql
+            transaction_item["params"] = tuple(param)
+            self.db_transaction.transaction_list.append(transaction_item)
+            return True
+        else:
+            list_row = self.db.insert(sql, tuple(param))
+            return list_row
 
-    def update_entity(self, model , field_list=None):
+    def update_entity(self, model, field_list=None):
         """
         @description: 根据模型的主键ID，更新字段的值
         @param model: 模型实体
         @return: 更新成功返回主键ID，否则返回空字符串
         @last_editors: ChenXiaolei
         """
-        field_list = self.model_obj.get_field_list()
+        if not field_list:
+            field_list = self.model_obj.get_field_list()
+        elif type(field_list) == str:
+            field_list = field_list.split(",")
         if len(field_list) == 0:
             return 0
         update_field_str = ""
@@ -325,11 +371,68 @@ class BaseModel:
         if mid == 0:
             return 0
         sql = f"UPDATE {self.table_name} SET {update_field_str} WHERE {self.primary_key_field}=%s;"
-        data = self.db.update(sql, tuple(param))
-        if data is not None and data.rowcount >= 0:
-            return mid
+        if self.is_transaction():
+            transaction_item = {}
+            transaction_item["sql"] = sql
+            transaction_item["params"] = tuple(param)
+            self.db_transaction.transaction_list.append(transaction_item)
+            return True
         else:
-            return ""
+            data = self.db.update(sql, tuple(param))
+            if data is not None and data > 0:
+                return mid
+            else:
+                return ""
+
+    def update_list(self, model_list, field_list=None, transaction=False):
+        """
+        @description: 根据模型的主键ID，更新字段的值
+        @param model: 模型实体
+        @return: 更新成功True  更新失败Flase
+        @last_editors: ChenXiaolei
+        """
+        if not model_list or len(model_list) == 0:
+            return False
+        sql_list = []
+        if not field_list:
+            field_list = self.model_obj.get_field_list()
+        elif type(field_list) == str:
+            field_list = field_list.split(",")
+        if len(field_list) == 0:
+            return 0
+        for model in model_list:
+            update_field_str = ""
+
+            param = []
+            mid = ""
+            for field_str in field_list:
+                if self.primary_key_field and str(field_str).lower() == self.primary_key_field.lower():
+                    mid = getattr(model, field_str)
+                    continue
+                update_field_str += str(field_str + "=%s,")
+                if str(field_str).lower() == "edit_on":
+                    now = datetime.datetime.now()
+                    param.append(str(now.strftime("%Y-%m-%d %H:%M:%S")))
+                else:
+                    param.append(str(getattr(model, field_str)))
+            param.append(mid)
+            update_field_str = update_field_str.rstrip(',')
+            if mid == 0:
+                continue
+            sql = f"UPDATE {self.table_name} SET {update_field_str} WHERE {self.primary_key_field}=%s;"
+            if self.is_transaction():
+                transaction_item = {}
+                transaction_item["sql"] = sql
+                transaction_item["params"] = tuple(param)
+                self.db_transaction.transaction_list.append(transaction_item)
+            elif transaction:
+                sql_list.append(sql)
+            else:
+                data = self.db.update(sql, tuple(param))
+        if transaction and not self.is_transaction():
+            if not self.transaction_execute(sql_list):
+                return False
+        return True
 
     def update_table(self, update_sql, where, params=None):
         """
@@ -343,11 +446,18 @@ class BaseModel:
         if where and where.strip() != '':
             where = " WHERE " + where
         sql = f"UPDATE {self.table_name} SET {update_sql}{where};"
-        data = self.db.update(sql, params)
-        if data is not None and data.rowcount >= 0:
+        if self.is_transaction():
+            transaction_item = {}
+            transaction_item["sql"] = sql
+            transaction_item["params"] = params
+            self.db_transaction.transaction_list.append(transaction_item)
             return True
         else:
-            return False
+            data = self.db.update(sql, params)
+            if data is not None and data > 0:
+                return True
+            else:
+                return False
 
     def get_dict(self, where='', group_by='', order_by='', limit='', field="*", params=None):
         """
@@ -432,12 +542,22 @@ class BaseModel:
         if where and where.strip() != '':
             where = " WHERE " + where
         if group_by and group_by.strip() != '':
-            group_by = " GROUP BY "+group_by
+            group_by = " GROUP BY " + group_by
         if order_by and order_by.strip() != '':
             order_by = " ORDER BY " + order_by
+
         sql = f"SELECT {field} FROM {self.table_name}{where}{group_by}{order_by} LIMIT {str(int(page_index) * int(page_size))},{str(page_size)}"
         list_row = self.db.fetch_all_rows(sql, params)
-        return list_row
+
+        sql = f"SELECT COUNT({self.primary_key_field}) AS count FROM {self.table_name}{where}{group_by};"
+        row = self.db.fetch_one_row(sql, params)
+
+        if row and 'count' in row and int(row['count']) > 0:
+            row_count = int(row["count"])
+        else:
+            row_count = 0
+
+        return list_row, row_count
 
     def transaction_execute(self, sql_list):
         """
@@ -449,3 +569,98 @@ class BaseModel:
         if not sql_list or len(sql_list) == 0:
             return False
         return self.db.transaction_execute(sql_list)
+
+    def is_transaction(self):
+        """
+        @description: 是否开启事务
+        @last_editors: ChenXiaolei
+        """
+        if hasattr(self, "db_transaction") and self.db_transaction and self.db_transaction.is_transaction == True:
+            return True
+        return False
+
+    def build_add_sql(self, model_list, ignore=False):
+        """
+        @description: 构造insert sql语句
+        @param model_list: 实体列表
+        @param ignore: 忽略已存在的记录
+        @return: sql语句字符串
+        @last_editors: ChenXiaolei
+        """
+        if not model_list or len(model_list) == 0:
+            return ""
+        build_sql = ""
+        field_list = self.model_obj.get_field_list()
+        if len(field_list) == 0:
+            return ""
+        for model in model_list:
+            insert_field_str = ""
+            insert_value_str = ""
+            for field_str in field_list:
+                param_value = str(getattr(model, field_str))
+                if str(field_str).lower() == self.primary_key_field.lower() and param_value == "0":
+                    continue
+                insert_field_str += str(field_str + ",")
+                insert_value_str += f"'{param_value}',"
+            insert_field_str = insert_field_str.rstrip(',')
+            insert_value_str = insert_value_str.rstrip(',')
+            sql = f"INSERT{' IGNORE' if ignore else ''} INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str});"
+            build_sql = build_sql+sql
+        return build_sql
+
+    def build_update_sql(self, model_list, field_list=None):
+        """
+        @description: 构造update sql语句
+        @param model_list: 实体列表
+        @param field_list: 需要更新的字段
+        @return: sql语句字符串
+        @last_editors: ChenXiaolei
+        """
+        if not model_list or len(model_list) == 0:
+            return ""
+        build_sql = ""
+        if not field_list:
+            field_list = self.model_obj.get_field_list()
+        elif type(field_list) == str:
+            field_list = field_list.split(',')
+        if len(field_list) == 0:
+            return 0
+        for model in model_list:
+            update_field_str = ""
+            for field_str in field_list:
+                if self.primary_key_field and str(field_str).lower() == self.primary_key_field.lower():
+                    continue
+                if str(field_str).lower() == "edit_on":
+                    update_field_str += f"{str(field_str)}='{TimeHelper.get_now_format_time()}',"
+                else:
+                    update_field_str += f"{str(field_str)}='{str(getattr(model, field_str))}',"
+
+            update_field_str = update_field_str.rstrip(',')
+            build_sql = build_sql + \
+                f"UPDATE {self.table_name} SET {update_field_str} WHERE {self.primary_key_field}='{getattr(model,self.primary_key_field)}';"
+        return build_sql
+
+    def build_data_sql(self, model_list):
+        """
+        @description: 构造replace sql语句
+        @param model_list: 实体列表
+        @return: sql语句字符串
+        @last_editors: ChenXiaolei
+        """
+        if not model_list or len(model_list) == 0:
+            return ""
+        build_sql = ""
+        field_list = self.model_obj.get_field_list()
+        if len(field_list) == 0:
+            return ""
+        for model in model_list:
+            insert_field_str = ""
+            insert_value_str = ""
+            for field_str in field_list:
+                insert_field_str += str(field_str + ",")
+                insert_value_str += f"'{str(getattr(model, field_str))}',"
+            insert_field_str = insert_field_str.rstrip(',')
+            insert_value_str = insert_value_str.rstrip(',')
+            sql = f"REPLACE INTO {self.table_name}({insert_field_str}) VALUE({insert_value_str});"
+            build_sql = build_sql+sql
+        return build_sql
