@@ -2,7 +2,7 @@
 """
 @Author: ChenXiaolei
 @Date: 2020-04-16 21:32:43
-@LastEditTime: 2020-05-15 20:27:22
+@LastEditTime: 2020-08-21 14:25:45
 @LastEditors: ChenXiaolei
 @Description: 
 """
@@ -83,9 +83,13 @@ class HTTPHelper:
         @last_editors: ChenXiaolei
         """
         try:
-            if hasattr(headers, "Content-Type") and headers["headers"].find("json") > 0:
-                return requests.post(url=url, json=data, headers=headers, **kwargs)
+            if "Content-Type" in headers and headers["Content-Type"].find("json") > 0:
+                if "json" in kwargs:
+                    return requests.post(url=url, headers=headers, **kwargs)
+                else:
+                    return requests.post(url=url, json=data, headers=headers, **kwargs)
+
             return requests.post(url=url, data=data, headers=headers, **kwargs)
         except:
-            print(f"get请求url:{url},params:{data}异常")
+            print(f"post请求url:{url},params-data:{data}异常")
             return None
